# -*- coding: utf-8 -*-
from django.test import TestCase
from django.template import Template, Context

class FilterTest(TestCase):

    def test_phone_filter_french(self):
        c = Context({"value": "+33.952400324"})
        t = Template("{% load intranet_values_extras %}{{ value|phone }}")
        self.assertEqual(t.render(c), '09 52 40 03 24')

    def test_phone_filter_international(self):
        c = Context({"value": "+41.952400324"})
        t = Template("{% load intranet_values_extras %}{{ value|phone }}")
        self.assertEqual(t.render(c), '+41952400324')

    def test_siret_filter(self):
        c = Context({"value": "01234567890128"})
        t = Template("{% load intranet_values_extras %}{{ value|siret }}")
        self.assertEqual(t.render(c), '012 345 678 90128')




# -*- coding: utf-8 -*-
from django.conf import settings
from django import template

register = template.Library()

@register.simple_tag
def intranet_title():
    return getattr(settings, 'INTRANET_TITLE', '')

@register.simple_tag
def title():
    return ' - '.join((getattr(settings, 'INTRANET_TITLE', ''),
                       getattr(settings, 'SITE_NAME', '')))

@register.assignment_tag(takes_context=True)
def has_perm(context, perm=None, prefix=False, user=None, obj=None):
    # If not user in param, get user in context
    if not user:
        user = context.get('user', None)
    # If not user => Not authorized
    if not user :
        return False

    # First case, `perm` in param
    if perm:
        app, perm_name = perm.split('.', 1)
        # Add prefix if in param
        if prefix:
            perm_name = u'%s_%s' % (prefix, perm_name)
        # If not app_label, add the context app_label
        if not app:
            app_label = context.get('app_label', None)
            if app_label:
                complete_perm = u'%s.%s' % (app_label, perm_name)
            else:
                complete_perm = perm_name
        else:
            complete_perm = perm_name
        # Check first global permissions
        is_authorized =  user.has_perm(complete_perm)
        # Check object permissions
        if not is_authorized and obj:
            is_authorized =  user.has_perm(perm_name, obj=obj)
        return is_authorized
    # Case #2 : not `perm` param but just `prefix`
    if prefix:
        module_name = context.get('module_name', None)
        if module_name:
            perm_name = u'%s_%s' % (prefix, module_name)
        else:
            perm_name = prefix
        app_label = context.get('app_label', None)
        if app_label:
            complete_perm = u'%s.%s' % (app_label, perm_name)
        else:
            complete_perm = perm_name
        is_authorized = user.has_perm(complete_perm)
        if not is_authorized and obj:
            is_authorized = user.has_perm(perm_name, obj)
        return is_authorized

    return False


@register.simple_tag
def default_table_class():
    return u'table table-striped table-bordered'



# -*- coding: utf-8 -*-

import datetime
import re

import floppyforms as forms

from django.db import models
from django.utils.translation import ugettext as _

from south.modelsinspector import add_introspection_rules
from intranet.templatetags.intranet_values_extras import phone as phone_render, siret as siret_render


################
# Phone Widget
################
EMPTY_VALUES = (None, '')
    
class PhoneInput (forms.TextInput):
    def render(self, name, value, attrs=None):
        if value not in EMPTY_VALUES:
            value = phone_render(value)
        else:
            value = phone_render('')

        return super(PhoneInput, self).render(name, value, attrs)

class PhoneFormField(forms.CharField):
    widget = PhoneInput

    def clean(self, value):
        phone = super(PhoneFormField, self).clean(value)

        #print "PHONE =", phone
        
        # If the number is not empty, we process the entered data.
        if phone != "":
            # Does the number contain a telephone prefix?
            if phone.startswith('+'):
                mo = re.search(r'^\+\d{2,3}\.\d{9,11}$', phone)
                
                if not mo:
                    raise forms.ValidationError(_(u'You must enter a telephone number. (+33.389520638 or 0389520638).'))
                else:
                    return phone
          
            # No telephone prefix: we are in France by default
            phone = re.sub("\D", "", phone) # Removal of separating characters
        
            mo = re.search(r'^\d{10}$', phone) # 10-digit number
            if not mo:
                raise forms.ValidationError(_(u'You must enter a telephone number. (+33.389520638 ou 0389520638).'))
            else:
                phone = mo.group()[-9:]
            
            return u'+33.%s' % phone
        
        # The number is empty, we don't do anything.
        else:
            return phone

class PhoneField(models.CharField):
    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 16
        super(PhoneField, self).__init__(*args, **kwargs)

    def get_internal_type(self):
        return "CharField"

    def formfield(self, form_class=PhoneFormField, **kwargs):
        return super(PhoneField, self).formfield(form_class=form_class, **kwargs)

add_introspection_rules([], ["^intranet\.widgets\.PhoneField"])

###################
# SIRET code widget
###################
    
class SiretInput(forms.TextInput):
    def render(self, name, value, attrs=None):
        if value not in EMPTY_VALUES:
            value = siret_render(value)
        else:
            value = phone_render('')

        return super(SiretInput, self).render(name, value, attrs)

class SiretFormField(forms.CharField):
    widget = SiretInput
    
    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 17
        super(SiretFormField, self).__init__(*args, **kwargs)

    def clean(self, value):
        
        if value in EMPTY_VALUES:
            return value
        else:
            siret = value.replace(' ','')
            if len(siret) != 14:
                raise forms.ValidationError(_(u'A SIRET number must contain exactly 14 digits.'))
            else:
                # Integer test
                try:
                    int(siret)
                except ValueError:
                    raise forms.ValidationError(_(u'A SIRET number must contain exactly 14 digits.'))
            
                # Validity test
                total = 0
                for i in range(14):
                    factor = (i % 2 == 0) and 2 or 1
                    itotal = factor * int(siret[i])
                    total += (itotal > 9) and itotal-9 or itotal
                    
                if total % 10 == 0:
                    return super(SiretFormField, self).clean(siret)
                else:
                    raise forms.ValidationError(_(u'The SIRET number you entered is invalid. Please enter a valid one.'))

class SiretField(models.CharField):
    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 14
        super(SiretField, self).__init__(*args, **kwargs)

    def get_internal_type(self):
        return "CharField"

    def formfield(self, form_class=SiretFormField, **kwargs):
        return super(SiretField, self).formfield(form_class=form_class, **kwargs)

add_introspection_rules([], ["^intranet\.widgets\.SiretField"])


# ------------------
# DatePicker Widget
# ------------------

class DatePicker(forms.TextInput):
    template_name = 'floppyforms/date.html'
    format="%d/%m/%Y"

    def render(self, name, value, attrs=None):
            if isinstance(value, datetime.date):
                    value=value.strftime(self.format)

            return super(DatePicker, self).render(name, value, attrs)
    
    # TODO: Dynamically load JS file instead of loading it in the template
    '''
    class Media:
        js = (
            '/_static/js/jquery-ui-timepicker-addon.js',
        )
    '''


# ------------------
# DateTimePicker Widget
# ------------------

class DateTimePicker(forms.TextInput):
    template_name = 'floppyforms/datetime.html'
    format="%d/%m/%Y %H:%M"

    def render(self, name, value, attrs=None):
            if isinstance(value, datetime.datetime):
                value=value.strftime(self.format)
            return super(DateTimePicker, self).render(name, value, attrs)
    
    # TODO: Dynamically load JS file instead of loading it in the template
    '''
    class Media:
        js = (
            '/_static/js/jquery-ui-timepicker-addon.js',
        )
    '''



# -*- coding: utf-8 -*-
from django.conf.urls import patterns, url
from django.contrib.auth.decorators import login_required
from django.forms import models as model_forms

from intranet.base_views import (IntranetListView, IntranetDetailView,
                                 IntranetCreateView, IntranetUpdateView,
                                 IntranetDeleteView)
from intranet.base_forms import IntranetModelForm

import floppyforms as forms

import logging
logger = logging.getLogger(__name__)


def get_default_model_url(model,
                          form_class=None,
                          url_name=None,
                          prefix_pattern='',
                          prefix_name='',
                          list_view=IntranetListView,
                          detail_view=IntranetDetailView,
                          creation_view=IntranetCreateView,
                          update_view=IntranetUpdateView,
                          delete_view=IntranetDeleteView):

    module_name = model._meta.module_name

    # Get default FormClass for model
    # e.g. : EntrepreneurForm for Entrepreneur model.
    if not form_class:
        form_class = model_forms.modelform_factory(model, IntranetModelForm)
    # Get default URL name
    if not url_name:
        url_name = module_name

    # Generation of urlpatterns
    urlpatterns = patterns('')
    if list_view:
        urlpatterns += patterns('',
                               # List View
                               url(r'^%s%s/$' % (prefix_pattern, module_name),
                                   login_required(list_view.as_view(model=model)),
                                   name='%s%s-list' % (prefix_name,url_name)))
    if detail_view:
        urlpatterns += patterns('',
                               # Detail View
                               url(r'^%s%s/(?P<pk>\d+)/$' % (prefix_pattern, module_name),
                                   login_required(detail_view.as_view(model=model)),
                                   name="%s%s-detail" % (prefix_name, url_name)))
    if creation_view:
        view_form_class = getattr(creation_view, 'form_class', None)
        if not view_form_class:
            view_form_class = form_class

        urlpatterns += patterns('',
                               # Add View
                               url(r'^%s%s/add/$' % (prefix_pattern, module_name),
                                   login_required(creation_view.as_view(model=model,
                                                                        form_class=view_form_class)),
                                   name="%s%s-add" % (prefix_name, url_name)))
    if update_view:
        view_form_class = getattr(update_view, 'form_class', None)
        if not view_form_class:
            view_form_class = form_class

        urlpatterns += patterns('',
                               # Edit View
                               url(r'^%s%s/(?P<pk>\d+)/edit/$' % (prefix_pattern, module_name),
                                   login_required(update_view.as_view(model=model,
                                                                      form_class=view_form_class)),
                                   name="%s%s-edit" % (prefix_name, url_name)))
    if delete_view:
        urlpatterns += patterns('',
                               # Delete View
                               url(r'^%s%s/(?P<pk>\d+)/delete/$' % (prefix_pattern, module_name),
                                   login_required(delete_view.as_view(model=model)),
                                   name="%s%s-delete" % (prefix_name, url_name)))

    return urlpatterns



# -*- coding: utf-8 -*-
import os
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse
from django.http import HttpResponse, Http404
from django.db.models import Q, BooleanField, NullBooleanField
from django.db.models.fields import FieldDoesNotExist
from django.views.generic import (TemplateView, ListView, 
                                  DetailView, CreateView,
                                  UpdateView, DeleteView)

# -----------------------
# INTRANET CONTEXT MIXINS
# -----------------------

class IntranetModelContextMixin(object):

    url_list = None
    url_detail = None
    url_add = None
    url_edit = None
    url_delete = None
    url_cancel = None

    def get_module_name(self):
        if self.model:
            return self.model._meta.module_name
        return ''

    def get_app_label(self):
        if self.model:
            return self.model._meta.app_label
        return ''

    def get_url(self, suffix):
        url = getattr(self, 'url_%s' % suffix, None)
        if url:
            return url
        else:
            return u'%s-%s' % (self.get_module_name(), suffix)

    def get_cancel_url(self):
        if self.url_cancel:
            return self.get_url(self.url_cancel)
        obj = getattr(self, 'object', None)
        if obj:
            return self.get_url('detail')
        return self.get_url('list')

    def get_urls(self):
        urls = {}
        for suffix in ['list', 'detail', 'add', 'edit', 'create', 'delete']:
            urls[suffix] = self.get_url(suffix)
        urls['cancel'] = self.get_cancel_url()
        return urls

    def get_success_url(self):
        obj = getattr(self, 'object', None)
        if obj:
            return reverse(self.get_url('detail'), args=[self.object.pk])
        return None

    def get_template_names(self):
        """
        Get the templates list to use for rendering.

        We first return the `template_name` attribute of the view,
        then the specific template of model (e.g. 'intranet/entrepreneur/model_list.html'),
        finaly, the generic template ('intranet/model_list.html').
        """
        app_label = self.model._meta.app_label
        module_name = self.model._meta.module_name
        base_template_name = u'model%s.html' % self.template_name_suffix

        template_list = []
        # Specific `template_name` of the view
        if self.template_name:
            template_list.append(self.template_name)
        # Specific template for app
        template_list.append(os.path.join(app_label, module_name, base_template_name))
        # Default template
        template_list.append(os.path.join('intranet', base_template_name))
        return template_list

    def get_context_data(self, **kwargs):
        context = super(IntranetModelContextMixin, self).get_context_data(**kwargs)
        # Add global template data
        model = getattr(self, 'model', None)
        if model:
            context['module_name'] = self.get_module_name()
            context['app_label'] = self.get_app_label()
            context['verbose_name'] = model._meta.verbose_name
            context['verbose_name_plural'] = model._meta.verbose_name_plural
        context['urls'] = self.get_urls()
        return context


# -----------------------
# INTRANET PERMS MIXINS
# -----------------------

class IntranetPermsMixin(object):
    perms = None
    prefix_perms = None
    perms_object = None
    perms_object_allowed = True

    def dispatch(self, request, *args, **kwargs):
        """
        Overload the super method and add checking rights access.

        If user in request has permissions required, normal response is returned.
        Else, the normal response is avorted and a response with a single message
        is returned.
        """
        # Try to dispatch to the right method; if a method doesn't exist,
        # defer to the error handler. Also defer to the error handler if the
        # request method isn't on the approved list.
        if request.method.lower() in self.http_method_names:
            handler = getattr(self, request.method.lower(), self.http_method_not_allowed)
        else:
            handler = self.http_method_not_allowed
        self.request = request
        self.args = args
        self.kwargs = kwargs
        # Check rights user
        if self.has_perms():
            return handler(request, *args, **kwargs)
        else:
            return HttpResponse('Sorry, access forbidden !')
       

    def get_user(self):
        return getattr(self.request, 'user', None)
        
    def get_perms(self):
        perms = []
        if self.perms:
            perms += self.perms
        if self.prefix_perms:
            if hasattr(self, 'get_app_label') and\
                    hasattr(self, 'get_module_name'):
                app_label = self.get_app_label()
                module_name = self.get_module_name()
                perms += ['%s.%s_%s' % (app_label, prefix, module_name)
                          for prefix in self.prefix_perms]
        return tuple(perms)

    def get_perms_object(self):
        perms_object = []
        if self.perms_object_allowed:
            if self.perms_object:
                perms_object += self.perms_object
            if self.prefix_perms:
                if hasattr(self, 'get_module_name'):
                    module_name = self.get_module_name()
                    perms_object += ['%s_%s' % (prefix, module_name)
                                     for prefix in self.prefix_perms]
        return tuple(perms_object)

    def get_object_for_perms(self):
        return self.get_object()

    def has_perms(self, *args, **kwargs):
        user = self.get_user()
        is_authorized = True
        perms = self.get_perms()
        perms_object = self.get_perms_object()
        # Perms required and not user
        # => Access denied
        if (perms or perms_object) and not user:
            return False
        # Global perms.
        if perms:
            for perm in perms:
                if not user.has_perm(perm):
                    is_authorized = False
                    break
        # Specific object perms.
        # Checked only if user is currently not allowed with global perms.
        if perms_object and (not is_authorized):
            obj = self.get_object_for_perms()
            for perm in perms_object:
                if user.has_perm(perm, obj):
                    is_authorized = True
                    break
        return is_authorized


# -------------------
# INTRANET VIEWS
# -------------------

class IntranetBaseView(IntranetPermsMixin, TemplateView):
    pass

class IntranetListView(IntranetModelContextMixin, ListView):
    list_display = None
    list_display_links = None
    search_fields = None
    list_filter = []

    search_url = ''
    ordering_url = ''
    filters_url = ''

    default_ordering_index = None
    default_ordering_asc = True


    def _append_to_url(self, url, param, value):
        if url:
            url += '&'
        url += u'%s=%s' % (param, value)

    def get_queryset(self):
        queryset = super(IntranetListView, self).get_queryset()
        if self.request.method == 'GET':
            q = self.request.GET.get('q', None)
            if q and self.search_fields is not None:
                self.search_url = 'q=%s' % q
                global_filter = Q()
                for field in self.search_fields:
                    searching_filter = u'%s__icontains' % field
                    global_filter |= Q(**{searching_filter:q})
                queryset = queryset.filter(global_filter)
        # LIST FILTERS
        self._apply_filters_params()
        if self.active_filters:
            query_filters = Q()
            for item, value in self.active_filters.items():
                filter_name = '%s__exact' % item
                query_filters &= Q(**{filter_name:value})
            queryset = queryset.filter(query_filters)
        # ORDERING FIELD
        self._apply_ordering_params()
        if self.ordering_field:
            # Model Field
            try:
                field = self.model._meta.get_field(self.ordering_field)
            except FieldDoesNotExist:
                field = None
            if field:
                ordering_str = self.ordering_field
            elif hasattr(self.model, self.ordering_field):
                attr = getattr(self.model, self.ordering_field)
                ordering_str = getattr(attr, 'admin_order_field', '')
            if ordering_str:
                if not self.ordering_asc:
                    ordering_str = '-%s' % ordering_str
                queryset = queryset.order_by(ordering_str)
            else:
                self.ordering_field = None

        if not self.ordering_field:
            default_ordering = self.model._meta.ordering
            print "DEFAULT ORDERING... :", default_ordering
            if default_ordering:
                queryset = queryset.order_by(*default_ordering)
        return queryset

    def _apply_ordering_params(self):
        self.ordering_field = None
        self.ordering_asc = True
        self.ordering_index = None
        if self.request.method != 'GET':
            return
        # Check if ordering asked
        ordering_params = self.request.GET.get('o')
        if ordering_params:
            self.ordering_url = 'o=%s' % ordering_params
            # Ordering Way
            if ordering_params.startswith('-'):
                self.ordering_asc = False
                ordering_params = ordering_params[1:]
            # Ordering Field
            try:
                self.ordering_index = int(ordering_params)-1
                self.ordering_field = self.list_display[self.ordering_index]
            except (ValueError, IndexError):
                # Disable ordering
                pass
        #elif self.list_display and self.default_ordering_index:
        
        #else:
        #    try:
        #        #print "-----------------------", self.default_ordering_index
        #        #print "-----------------------", self.list_display
        #        self.ordering_field = self.list_display[self.default_ordering_index]
        #        self.ordering_index = self.default_ordering_index
        #        self.ordering_asc = self.default_ordering_asc
        #    except (IndexError, TypeError):
        #        print "EXCEPTION...."
        #        self.ordering_field = None
        #        self.ordering_index = None
        
        # elif self.list_display:
        #    # By Default, we order by the first field displayed
        #    self.ordering_index = 0
        #    self.ordering_field = self.list_display[self.ordering_index]

    def _apply_filters_params(self):
        self.active_filters = {}
        if self.request.method != 'GET' and self.list_filter:
            return
        for field in self.list_filter:
            value = self.request.GET.get(field, '')
            if value != '':
                self.active_filters[field] = value
        self.filters_url = '&'.join(['%s=%s' % (key, value)
                                     for key, value in self.active_filters.items()])

    def get_context_list_display(self):
        # Current Url with search and filters params
        params_url_items = []
        if self.search_url:
            params_url_items.append(self.search_url)
        if self.filters_url:
            params_url_items.append(self.filters_url)
        params_url = '&'.join(params_url_items)
        if params_url:
            current_url = '?%s&' % params_url
        else:
            current_url = '?'
        # --
        # Check if fields are valid
        context_list_display = []
        model_fields_list = self.model._meta.get_all_field_names()
        if self.list_display is not None:
            for field in self.list_display:
                item = {}
                if field in model_fields_list:
                    model_field = self.model._meta.get_field(field)
                    item['verbose_name'] = model_field.verbose_name
                    # Boolean 'on'/'off' displaying
                    if isinstance(model_field, BooleanField) or\
                            isinstance(model_field, NullBooleanField):
                        item['boolean'] = True
                elif hasattr(self.model, field):
                    attr = getattr(self.model, field)
                    item['callable'] = True
                    if hasattr(attr, 'short_description'):
                        item['verbose_name'] = getattr(attr, 'short_description')
                    else:
                        item['verbose_name'] = attr.__name__
                    # Boolean 'on'/'off' displaying
                    item['boolean'] = getattr(attr, 'boolean', False)
                if item:
                    item['field'] = field
                    if self.ordering_field == field:
                        item['ordering'] = True
                        item['ordering_asc'] = self.ordering_asc
                        if self.ordering_asc:
                            item['ordering_url'] = '%so=-%s' % (current_url, self.ordering_index+1)
                        else:
                            item['ordering_url'] = '%so=%s' % (current_url, self.ordering_index+1)
                    else:
                        item['ordering_url'] = '%so=%s' % (current_url, self.list_display.index(field)+1)
                    context_list_display.append(item)
                else:
                    raise ImproperlyConfigured("%s : Model `%s` hasn't field named `%s`." % (
                            self.__class__.__name__, self.model.__name__, field))
        return context_list_display

    def get_context_list_filter(self):
        context_list_filter = []
        # Current url with search and ordering params
        params_url = []
        if self.search_url:
            params_url.append(self.search_url)
        if self.ordering_url:
            params_url.append(self.ordering_url)
        # --
        for field in self.list_filter:
            infos = {}
            # Verbose Name
            model_fields_list = self.model._meta.get_all_field_names()
            if field in model_fields_list:
                verbose_name = self.model._meta.get_field(field).verbose_name
            elif hasattr(self.model, field):
                attr = getattr(self.model, field)
                if hasattr(attr, 'short_description'):
                    verbose_name = getattr(attr, 'short_description')
                else:
                    verbose_name = attr.__name__
            infos['verbose_name'] = verbose_name
            # URLs
            params_filters_url = '&'.join(['%s=%s' % (key, value)
                                          for key, value in self.active_filters.items()
                                          if key != field])
            if params_filters_url:
                global_params_url = params_url + [params_filters_url]
            else:
                global_params_url = list(params_url)
            infos['url'] = {'none': '?%s' % '&'.join(global_params_url),
                            'yes': '?%s=1' % '&'.join(global_params_url+[field]),
                            'no': '?%s=0' % '&'.join(global_params_url+[field])}
            # Current Value
            if field in self.active_filters.keys():
                infos['value'] = self.active_filters[field]
            context_list_filter.append(infos)
        return context_list_filter
        
    def get_context_data(self, **kwargs):
        context = super(IntranetListView, self).get_context_data(**kwargs)
        context['list_display'] = self.get_context_list_display()
        context['list_display_links'] = self.list_display_links
        context['search_fields'] = self.search_fields
        context['list_filter'] = self.get_context_list_filter()
        context['active_filters'] = [(key, value) for key, value in self.active_filters.items()]
        return context

class IntranetDetailView(IntranetPermsMixin, IntranetModelContextMixin, DetailView):
    def get_list_queryset(self):
        STEP = 5
        LIST_SIZE = 2*STEP+1
        current_object = self.get_object()
        print 'oooooooooooo', self.model._meta.ordering
        query_set = self.model.objects.order_by(*self.model._meta.ordering)
        index = list(query_set).index(current_object)
        
        # Index too close to start
        if index < STEP:
            start_ind = 0
            end_ind = LIST_SIZE # We don't care if it's too big.
        # Index too close to end
        elif index+STEP >= len(query_set):
            end_ind = len(query_set)
            start_ind = end_ind>=LIST_SIZE and end_ind-LIST_SIZE or 0
        else:
            start_ind = index-STEP
            end_ind = start_ind+LIST_SIZE
        
        return query_set[start_ind:end_ind]

    def get_context_data(self, **kwargs):
        context = super(IntranetDetailView, self).get_context_data(**kwargs)
        context['object_list'] = self.get_list_queryset()
        return context


class IntranetCreateView(IntranetPermsMixin, IntranetModelContextMixin, CreateView):
    url_cancel = 'list'
    perms_object_allowed = False
    prefix_perms = ('add',)


class IntranetUpdateView(IntranetPermsMixin, IntranetModelContextMixin, UpdateView):
    url_cancel = 'detail'
    prefix_perms = ('change',)


class IntranetSectionUpdateView(IntranetUpdateView):
    section = None
    sections_filter = None

    def dispatch(self, *args, **kwargs):
        # Save the section model
        self.section = kwargs.get('section', None)
        # If filter sections, we check if
        # this section is allowed.
        if self.section is not None and\
                self.sections_filter is not None:
            if self.section not in self.sections_filter:
                raise Http404
        # Default Process
        return super(IntranetSectionUpdateView, self).dispatch(*args, **kwargs)

    def get_form_class(self):
        if self.section is not None:
            form_class_name = u'%sSection%sForm' % ( self.model.__name__, self.section.capitalize())
            module_name = '%s.forms' % self.model.__module__.rsplit('.', 1)[0]
            try:
                module = __import__(module_name, fromlist=[form_class_name])
                form_class = getattr(module, form_class_name, None)
                if form_class:
                    return form_class
            except ImportError:
                pass
        # Default Case
        return super(IntranetSectionUpdateView, self).get_form_class()

    def get_context_data(self, **kwargs):
        context = super(IntranetSectionUpdateView, self).get_context_data(**kwargs)
        context['section'] = self.section
        return context

class IntranetDeleteView(IntranetPermsMixin, IntranetModelContextMixin, DeleteView):
    url_cancel = 'detail'
    prefix_perms = ('delete',)

    def get_success_url(self):
        return reverse(self.get_url('list'))



# -*- coding: utf-8 -*-
from django.utils.translation import ugettext as _

def make_btn(url, **kwargs):
    """
    Easy generating of Bootstrap link (with button style).
    
    Kwargs available are the following :
      - css : CSS style of the button. Allowed values : 'primary', 'info', 'success',
              'warning', 'danger' or 'inverse'. By default, display a standard gray button.
      - size : Size of the button. Allowed values are : 'large', 'small' or 'mini'.
               By default, use the normal size.
      - align : 'right' or 'left'. By default is None
      - type : Predefined Configuration of txt and icon.
               Allowed values : 'edit'.
      - txt : Text value of the button
      - icon : Suffix of a Glyphicons label (e.g. 'edit', 'music', ...)
      - icon_white : True if you want white icon (default = False)
    """
    # Initialization
    item_class = 'btn'
    item_txt = 'Action'
    item_icon_type = item_icon_white = item_icon = ''
    # Make class button
    if 'css' in kwargs:
        item_class += ' btn-%s' % kwargs['css']
    if 'size' in kwargs:
        item_class += ' btn-%s' % kwargs['size']
    if 'align' in kwargs:
        item_class += ' pull-%s' % kwargs['align']
    item_class = u' class="%s"' % item_class
    # Standard Configurations
    std_type = kwargs.get('type', None)
    if std_type == 'edit':
        item_txt = _(u'Edit')
        item_icon_type = 'edit'
    # elif type == '':
    #     item_txt = 
    #     item_icon_type =
    # Make icon white ?
    if kwargs.get('icon_white', False):
        item_icon_white = ' icon-white'
    # Customization
    if 'text' in kwargs:
        item_txt = kwargs['txt']
    if 'icon' in kwargs:
        item_icon_type = kwargs['icon']
    # Make icon
    if item_icon_type:
        item_icon = '<i class="icon-%s%s"></i>&nbsp;' % (item_icon_type, item_icon_white)
    # Make button
    button = '<a%s href="%s">%s%s</a>' % (item_class, url,
                                          item_icon, item_txt)
    return button



# -*- coding: utf-8 -*-
from django.test import TestCase
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as _
from django.conf import settings
from django.contrib.auth.models import User

from intranet.tests import HOST_SERVER_TEST, default_login_with_user

class LoginTest(TestCase):

    fixtures = ['intranet_users.yaml']

    def setUp(self):
        # User Admin : superuser
        self.user_admin = User.objects.get(username='admin')
        # User 0 : all rights (e.g. 'permanent' )
        self.user_0 = User.objects.get(username='user0')
        # User 1 : no rights
        self.user_1 = User.objects.get(username='user1')
        # User 2 : no rights
        self.user_2 = User.objects.get(username='user2')
        # User 3 : no rights
        self.user_3 = User.objects.get(username='user3')
        # User 4 : no rights
        self.user_4 = User.objects.get(username='user4')


    def test_login_required_view_with_anonymous_user(self):
        """
        Test a simple login required view with an non loged user.

        We get a 302 status code response and redirect to the login url.
        """
        url = reverse('intranet:dashboard')
        response = self.client.get(url, follow=True)
        # We check redirection to login page
        expected_url = u'%s%s?next=%s' % (HOST_SERVER_TEST, settings.LOGIN_URL, url)
        self.assertRedirects(response, expected_url)


    def test_login_required_view_with_loged_superuser(self):
        """
        Test a simple login required view with an loged superuser.

        We get a 200 status code response and the dashboard view.
        """
        url = reverse('intranet:dashboard')
        # Authentication of user
        is_authorized = default_login_with_user(self.client, self.user_admin)
        self.assertEqual(is_authorized, True)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_login_required_view_with_loged_user(self):
        """
        Test a simple login required view with an loged user (no superuser).

        We get a 200 status code response and the dashboard view.
        """
        # Authentication of entrepreneur
        url = reverse('intranet:dashboard')
        # Authentication of user
        is_authorized = default_login_with_user(self.client, self.user_0)
        self.assertEqual(is_authorized, True)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)



# -*- coding: utf-8 -*-
from django.conf import settings
from django import template

from intranet.components import make_btn

register = template.Library()

@register.simple_tag
def btn(**kwargs):
    return make_btn(**kwargs)



# -*- coding: utf-8 -*-
from django.conf import settings
from django.utils.translation import ugettext as _
from django import template

register = template.Library()

@register.simple_tag(takes_context=True)
def list_header(context, list_display=None):
    if not list_display:
        list_display = context.get('list_display', [])
    header = ''
    if list_display:
        for field in list_display:
            ordering = field.get('ordering', False)
            if ordering:
                if field.get('ordering_asc', False):
                    ordering_icon = 'icon-chevron-down'
                else:
                    ordering_icon = 'icon-chevron-up'
                ordering_html = '<span class="pull-right"><i class="%s"></i></span>' % (
                    ordering_icon)
            else:
                ordering_html = ''

            if field.get('callable', False) and 'short_description' in field:
                field_verbose_name = field.getattr('short_description')
            else:
                field_verbose_name = field['verbose_name'].capitalize()
                
            header += u'<th><a href="%s">%s%s</a></th>' % (field.get('ordering_url', ''),
                                                           field_verbose_name,
                                                           ordering_html)
    else:
        header = '<th>%s</th>' % context.get('verbose_name_plural', '').capitalize()
    return u'<tr>%s</tr>' % header


@register.simple_tag(takes_context=True)
def list_item(context, list_display=None, list_display_links=None, obj=None, url=''):
    # PARAMS initialization
    if not list_display:
        list_display = context.get('list_display', [])
    if not obj:
        obj = context.get('object', None)
    if not list_display_links:
        list_display_links = context.get('list_display_links', [])
    if not url:
        url = context.get('url_object_detail', '')
    # Rendering item
    item = ''
    if obj:
        if list_display:
            if not list_display_links:
                list_display_links = list_display[0]['field']
            for field in list_display:
                attr = getattr(obj, field['field'])
                if field.get('callable', False):
                    attr = attr()
                if field.get('boolean', False):
                    if attr:
                        boolean_icon = 'icon-ok'
                        boolean_title = _('Yes')
                    else:
                        boolean_icon = 'icon-remove'
                        boolean_title = _('No')
                    attr = '<i title="%s" class="%s"></i>' % (boolean_title, boolean_icon)
                if url and field['field'] in list_display_links:
                    item += u'<td><a href="%s">%s</a></td>' % (url, attr)
                else:
                    item += u'<td>%s</td>' % attr
        else:
            # If no list_display, we display the default verbose name object
            if url:
                item = u'<td><a href="%s">%s</a></td>' % (url, obj)
            else:
                item = u'<td>%s</td>' % (obj)
    if item:
        return u'<tr>%s</tr>' % item
    return ''



# -*- coding: utf-8 -*-
from django.contrib.auth.forms import AuthenticationForm, PasswordChangeForm
from intranet.base_forms import IntranetForm, IntranetModelForm


class LoginIntranetForm(IntranetForm, AuthenticationForm):
    pass

class PasswordChangeIntranetForm(IntranetForm, PasswordChangeForm):
    
    def __init__(self, user, *args, **kwargs):
        super(PasswordChangeIntranetForm, self).__init__(user, *args, **kwargs)



# -*- coding: utf-8 -*-

HOST_SERVER_TEST = 'http://testserver' 

def default_login_with_user(client, user):
    return client.login(username=user.username,
                        password=user.username)

from intranet.tests.filters import *




from django import template

register = template.Library()

import re

@register.filter(name='phone')
def phone(value):
    """
    Transform a number "+81.0123456789" into "+810123456789" and a number "+33.384289468" into "03 84 28 94 68" for better readablility.
    """

    if value.startswith('+33.'):
        # get 9 or 10 digits, or None:
        mo = re.search(r'\d{9,10}', value)

        # If the number is not standardized, we don't modify it.
        if mo is None: return value
        
        # add a leading 0 if they were just 9
        digits = mo.group().zfill(10)
        
        # now put a dot after each 2 digits
        return " ".join(re.findall(r'(\d\d)',digits))
    else:
        return re.sub(r'\+(\d{2,3})\.(\d{9,11})', r"+\1\2", value)

@register.filter(name='siret')
def siret(value):
    """
    Transform a SIRET code "73282932000074" into "732 829 320 00074" for better readability.
    """
    
    if value=='' :
        return ''
    else:
        return re.sub(r'(\d{3})(\d{3})(\d{3})(\d{5})', r'\1 \2 \3 \4', value)




# -*- coding: utf-8 -*-
from django.db import models
from django.contrib.auth.models import User
from django.db.models.signals import post_save

class UserProfile(models.Model):
    user = models.ForeignKey(User, unique=True)

    @property
    def dashboard(self):
        return None

def createUserProfile(sender, instance, **kwargs):
    """Create a UserProfile object each time a User is created ; and link it.
    """
    UserProfile.objects.get_or_create(user=instance)

post_save.connect(createUserProfile, sender=User)



# -*- coding: utf-8 -*-
from django.conf import settings
from django.conf.urls import patterns, include, url
from django.core.urlresolvers import reverse_lazy
from django.contrib.auth.decorators import login_required
from intranet.views import (DashboardView)
from intranet.forms import LoginIntranetForm, PasswordChangeIntranetForm


urlpatterns = patterns('',

                       # Dashboard
                       url(r'^dashboard/$', login_required(DashboardView.as_view()), name="dashboard"),

                       # Log In/Out URLs
                       url(r'^accounts/login/$', 'django.contrib.auth.views.login',
                           {'template_name': 'intranet/registration/login.html',
                            'authentication_form': LoginIntranetForm,},
                           name="login"),
                       url(r'^accounts/logout/$', 'django.contrib.auth.views.logout_then_login',
                           name="logout"),
                       url(r'^accounts/password/change/$', 'django.contrib.auth.views.password_change',
                           {'template_name': 'intranet/registration/password_change.html',
                            'password_change_form' : PasswordChangeIntranetForm,
                            'post_change_redirect': settings.PASSWORD_CHANGE_REDIRECT_URL,
                            'extra_context': {'cancel_url': settings.PASSWORD_CHANGE_REDIRECT_URL}},
                           name="password-change"),

                       )
